package com.flyfish.framework.domain.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.enums.UserStatus;
import com.flyfish.framework.enums.UserType;
import lombok.*;
import org.springframework.data.annotation.Transient;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.Date;
import java.util.List;

@Document
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class User extends AuditDomain implements IUser {

    private static final long serialVersionUID = -960011918745179950L;

    /**
     * 用户类型
     */
    private UserType type;

    /**
     * 用户状态
     */
    private UserStatus status;

    /**
     * 冗余的电话号码
     */
    private String phone;

    /**
     * 用户名
     */
    @Indexed(unique = true)
    private String username;

    /**
     * 密码
     */
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private String password;

    /**
     * 是否启用
     */
    private Boolean enable;

    /**
     * 能否登录移动端
     */
    private Boolean app;

    /**
     * 有效期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date validDate;

    /**
     * 可操作校区
     */
    @DBRef
    private List<Department> departments;

    /**
     * 所属角色
     */
    @DBRef
    private List<Role> roles;

    /**
     * 微信openId
     */
    private String openId;

    /**
     * 查询冗余，标记用户信息
     */
    @Transient
    private Object detail;

    /**
     * 错误次数，防止恶意登录
     */
    private Integer errorCount = 0;

    @Override
    public User toUser() {
        return this;
    }
}
