package com.flyfish.framework.logging.advice;

import com.flyfish.framework.logging.service.LogManager;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;

import javax.annotation.Resource;

/**
 * 日志切面
 *
 * @author wangyu
 */
@Aspect
public class LogAdvice {

    @Resource
    private LogManager logManager;

    /**
     * 切入注解
     */
    @Pointcut("execution(public * com.flyfish..*.*Controller.*(..))")
    public void pointCut() {

    }


    /**
     * 环绕切面，获取参数
     *
     * @param joinPoint 切入点
     * @return 结果
     */
    @Around("pointCut()")
    public Object process(ProceedingJoinPoint joinPoint) throws Throwable {
        Object[] args = joinPoint.getArgs();
        Throwable e = null;
        Object result = null;
        try {
            result = joinPoint.proceed(args);
            return result;
        } catch (Throwable throwable) {
            e = throwable;
            throw throwable;
        } finally {
            logManager.tryLog(joinPoint, result, e);
        }
    }
}
